/*
*  qm_player.h
*  QUIMUP main player window
*  © 2008-2024 Johan Spee
*  SPDX-License-Identifier: GPL-3.0-or-later
*/

#ifndef QM_PLAYER_H
#define QM_PLAYER_H

#include <QtWidgets>
#include <QApplication>
#include <QMainWindow>
#include <QCloseEvent>
#include <QHideEvent>
#include <QShowEvent>
#include <QFrame>
#include <QLabel>
#include <QPushButton>
#include <QSlider>
#include <QImage>
#include <QPixmap>
#include <QSettings>
#include <QTimer>
#include <QSignalMapper>
#include <QPalette>
#include <QObject>
#include <QDir>
#include <QProcess>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <QMenu>
#include <QWidgetAction>
#include <stdlib.h>
#include "qm_clicklabel.h"
#include "qm_clickprogressbar.h"
#include "qm_config.h"
#include "qm_mpdcom.h"
#include "qm_trayicon.h"
#include "qm_scroller.h"
#include "qm_settings.h"
#include "qm_browser.h"
#include "qm_songinfo.h"

typedef struct qm_fonts
{
    int  title_size = 0;
    bool title_italic = false;
    bool title_bold = false;
    int  codecinfo_size = 0;
    bool codecinfo_bold = false;
    int  time_size = 0;
    bool time_bold = false;
    int  album_size = 0;
    bool album_bold = false;
    int  year_size = 0;
    bool year_bold = false;
    int  browser_size = 0;
    int  comment_size = 0;
    bool comment_italic = false;
}
qm_fonts;

class qm_Player : public QMainWindow
{
    Q_OBJECT

public:
    qm_Player(qm_Config *);
    virtual ~qm_Player();

    qm_Browser *browser_window;

protected:
    void keyPressEvent(QKeyEvent *);

public slots:
    void on_shutdown();
    void wakeup_call(bool);

private slots:
    void on_contextmenu();
    void on_mpdconnect_sgnl(bool, bool);
    void set_status(int, QString s = "");
    void on_signal(int);
    void on_new_song();
    void on_new_status(mpd_status *);
    void on_tray_clicked();
    void lock_progress(QMouseEvent *e);
    void unlock_progress();
    void maximice();
    void minimice();
    void show_saved_albumart();
    void open_albumart_dir();
    void open_directory();
    void edit_tags();
    void reload_tags();
    void set_colors();
    void set_fonts();
    void on_new_custom_art();
    void on_curentsong_deleted();
    void on_volumedial(int);
    void on_browser_keypress(QKeyEvent *);
    void config_external_apps();

private:
    QWidget *main_widget;
    QVBoxLayout *vbox_all;

    QLabel *main_display;
    QVBoxLayout *vbox_in_main_display;
    qm_Scroller *info_scroller;
    QHBoxLayout *hbox_info;
    QLabel *lb_extension;
    QLabel *lb_kbps;
    QLabel *lb_kHz;
    QSpacerItem *spacer_info;
    qm_clickLabel *lb_time;
    qm_clickProgressbar *pb_progress;
    QSpacerItem *spacer_main;

    QLabel *center_widget;
    QHBoxLayout *hbox_center;
    QLabel *lb_albumart;

    QVBoxLayout *vbox_cright;
    QLabel *lb_album;
    QLabel *lb_year;
    QScrollArea *sa_comment;
    QLabel *lb_comment;

    QHBoxLayout *hbox_bottom;
    QPushButton *bt_prev;
    QPushButton *bt_stop;
    QPushButton *bt_playpause;
    QPushButton *bt_next;
    QPushButton *bt_browser;
    QSlider *vol_slider;
    QPushButton *bt_sizer;
    QPushButton *bt_setts;

    qm_Settings * settings_window;
    QSignalMapper *button_mapper;
    qm_trayIcon *the_trayicon;
    QTimer *maximizer;
    QTimer *minimizer;
    qm_Config *config;
    qm_mpdCommand *mpd_cmd;
    qm_songInfo *current_songinfo;
    qm_fonts *current_fonts;

    QString
    pb_stylesheet,
    current_mpd_dir,
    current_art_path;  // used when colors change

    int
    current_type,
    current_status,
    current_volume,
    current_song_pos,
    kbps_count_down,
    alphacycler,
    albumart_H,
    albumart_W,
    song_total_time,
    song_previous_time,
    cR, cG, cB,
    window_H,
    window_H_max,
    window_W_max;

    bool
    b_repeat,
    b_random,
    b_single,
    b_consume,
    b_xfading,
    b_minmax_fading,
    b_minmax_busy,
    b_mpd_sets_volume,
    b_stream,
    b_mpd_connected,
    b_settings_hidden,
    b_browser_hidden,
    b_nosong,
    b_lock_progress,
    b_really_close,
    b_use_trayicon,
    b_need_new_song,
    b_reshow_browser,
    b_reshow_settings,
    b_skip_khz,
    b_wayland,
    b_remote_server;

    QPixmap
    px_albmart,
    px_xfading;

    QImage
    im_nopic,
    im_stream,
    im_fading_input,
    im_albumart;

    QIcon
    ic_bt_prev,
    ic_bt_stop,
    ic_bt_next,
    ic_bt_browser,
    ic_bt_sizer,
    ic_bt_setts,
    ic_bt_pp_pause,
    ic_bt_pp_play,
    ic_a_tagedit,
    ic_a_opendir,
    ic_a_openaart,
    ic_a_loadtags;


    QMenu * context_menu;
    QAction
    *a_openaart,
    *a_tagedit,
    *a_loadtags,
    *a_opendir;

    void setup_widgets();
    void set_themed_icons(bool);
    void retranslate();
    void init_vars();
    QString into_time(int);
    void get_and_set_albumart(int, QString);
    bool get_art_from_local_file(int, QString);
    void set_comment_text(QString);
    void set_album_text(QString);
    void set_year_text(QString);
    bool is_app_installed(QString);

protected:
    void closeEvent(QCloseEvent *);
    void hideEvent(QHideEvent *);
    void showEvent(QShowEvent *);
    void dragLeaveEvent(QDragLeaveEvent *);
    void dragEnterEvent(QDragEnterEvent *);
    void dragMoveEvent(QDragMoveEvent *);
    void dropEvent(QDropEvent *);
    // void moveEvent(QMoveEvent *); // never happens on wayland
};

#endif // QM_PLAYER_H
